# Gitlab CI Runner

## Description

Install and configure *runners* for **gitlab-ci**. Use
**gitlab-ci-multi-runner** as runner type. Learn more on
<https://gitlab.com/gitlab-org/gitlab-ci-multi-runner>.

## Requirements

### Cookbooks and gems

Declared in [metadata.rb](metadata.rb) and in [Gemfile](Gemfile).

### Platforms

- RHEL Family 7, tested on CentOS
- Debian 8

## Usage

### Easy Setup

Add `recipe[gitlab-ci-runner]` in your run-list to install
**gitlab-ci-multi-runner** using its repository.

To add runners, use default provider or use
`recipe[gitlab-ci-runner::register]` and `runners` attribute. To see a use
case, look at [kitchen.yml](kitchen.yml).

## Test

This cookbook is fully tested through the installation of the full gitlab
platform in docker hosts. This uses kitchen, docker and some monkey-patching.

If you run kitchen list, you will see 3 suites, Each corresponds to a different
server:

- kitchen-gitlab-ubuntu-1404: complete instance of **gitlab-ce**
- kitchen-gitlab-ci-runner-centos-7: installation of **gitlab-ci-multi-runner**
  and configuration of runners on a centos-7
- kitchen-gitlab-ci-runner-debian-8: same but for debian-8

For more information, see [kitchen.yml](kitchen.yml) and [test](test)
directory.

## Recipes

### default

Install **gitlab-ci-multi-runner** by using its repository, enable and start
the service.

### repository

Install runner repository.

### package

Install **gitlab-ci-multi-runner** package.

### service

Enable and start runner service.

### register

Register or Unregister runners based on attribute
`['gitlab-ci-runner']['runners']`. This is an Array of Hash where each Hash
represent an action on a runner, "register" or "unregister". In a Hash,
`description` key is the name attribute and represents the name of the runner.
Then each runner option should be listed in a sub-Hash of key `options`, each
sub-key matching an option name of gitlab-runner cli.

**NOTICE:** Due to the way that the cache configuration file parameter names
do not match their command line variants in a reliably convertible way they
will not be set during initial registration. They will be added to the
configuration file during the next chef run. You are advised to use the
`register_string` property to manually set the registration string if this
will be an issue.

See [kitchen.yml](kitchen.yml) for an example.

## Resources

### gitlab_ci_runner_global_config

Manage the global options of Gitlab CI runner, all settings are optional.

```ruby
gitlab_ci_runner_global_config '/etc/gitlab-runner/config.toml' do
  concurrent 5
  log_level info
  log_format text
  check_interval 0
  sentry_dsn false
  listen_address '127.0.0.1'
  session_server(
    "listen_address" => "0.0.0.0:8093",
    "advertise_address" => "runner-host-name.tld:8093",
    "session_timeout"=> 1800
  )
end
```

The full resource options list is available in
[resources/global_config.rb](resources/global_config.rb).

### gitlab_ci_runner_package

Manage the installation of Gitlab CI runner, all settings are optional.

```ruby
gitlab_ci_runner_package 'gitlab-runner' do
  repo_url 'http://specify.custom.repo'
  gpgkey 'http://specify.custom.repo/gpgkey'
  gpgcheck false
  version '12.7.1'
  package_options(
    'retries': 3,
    'retry_delay': 5
  )
  action :install
end
```

The full resource options list is available in
[resources/global_config.rb](resources/package.rb).

### gitlab_ci_runner_service

Manage the Gitlab CI runner service, all settings are optional.

```ruby
gitlab_ci_runner_service 'gitlab-runner' do
  action [:enable, :start]
end
```

The full resource options list is available in
[resources/global_config.rb](resources/service.rb).

### gitlab_ci_runner_runner

#### actions

- `:register` - Register a runner updating its settings if already registered.
- `:update` - Update settings for a previously registered runner.
- `:unregister` - Unregister a runner and remove its configuration.

#### General properties

| Property               | Optional?       | Type   |
|------------------------|-----------------|--------|
| `url`                  | Register: No    | String |
|                        | Unregister: Yes |        |
| `registration_token`   | Register: No    | String |
|                        | Unregister: Yes |        |
| `administration_token` | Yes             | String |
| `config`               | Yes             | String |

#### Server option properties

| Property          | Optional?  | Type   |
|-------------------|------------|--------|
| `description`     | Yes        | String |
| `active`          | Yes        | Bool   |
| `tag_list`        | Yes        | Array  |
| `run_untagged`    | Yes        | Bool   |
| `locked`          | Yes        | Bool   |
| `access_level`    | Yes        | String |
| `maximum_timeout` | Yes        | Int    |

#### Local option properties

| Property  | Optional?       | Type   |
|-----------|-----------------|--------|
| `options` | Yes             | Hash   |

Register or Unregister runners. For instance, to register a default runner
to <http://gitlab-ci.myinstance>:

```ruby
gitlab_ci_runner_runner 'my runner' do
  registration_token '1234567890'
  administrative_token '987654321'
  url 'http://gitlab-ci.myinstance'
  tag_list [ "testing-1", "testing-2" ]
  options({
    'executor': 'shell'
  })
end
```

Runners that are already registered will have their configuration updated to
match the supplied options. Empty hash values will be compacted (removed).

If an administrative token with suitable permissions has been provided the
server runner configuration will also be updated to match the resource.

Example options:

```ruby
{
  "name" => "Testing Docker Runner 1",
  "url" => "https://gitlab.test.com",
  "registration_token" => "1234567890",
  "options" => {
    "executor" => "docker",
    "docker" => {
      "image" => "centos",
      "privileged" => false,
      "disable_entrypoint_overwrite" => false,
      "oom_kill_disable" => false,
      "disable_cache" => false,
      "volumes" => [
        "/cache",
        "/etc/chef:/etc/chef:ro"
      ],
      "shm_size" => 0
    },
    "cache" => {
      "s3" => {},
      "gcs" => {}
    },
    "custom_build_dir" => {
      "enabled" => true
    }
  },
  "tag_list" => [
    "docker-test-1"
  ]
}
```

Will generate this configuration:

```toml

[[runners]]
executor = "docker"
name = "Testing Docker Runner"
token = "9dK52sYkuXeVudzw3HuH"
url = "https://gitlab.test.com"

[runners.docker]
disable_cache = false
disable_entrypoint_overwrite = false
image = "centos"
oom_kill_disable = false
privileged = false
shm_size = 0
volumes = ["/cache","/etc/chef:/etc/chef:ro"]

[runners.custom_build_dir]
enabled = true
```

The full resource options list is available in
[resources/runner.rb](resources/runner.rb).

### gitlab_ci_runner_global_config

#### Actions

- `:set` - Get runner global configuration options

#### Properties

| Property               | Optional?       | Type   |
|------------------------|-----------------|--------|
| `config`               | Yes             | String |
| `concurrent`           | Yes             | Int    |
| `log_level`            | Yes             | String |
| `log_format`           | Yes             | String |
| `check_interval`       | Yes             | Int    |
| `sentry_dsn`           | Yes             | Bool   |
| `listen_address`       | Yes             | String |
| `options`              | Yes             | Hash   |

Manage the global options of Gitlab CI runner, all settings are optional.

```ruby
gitlab_ci_runner_global_config '/etc/gitlab-runner/config.toml' do
  concurrent 5
  log_level info
  log_format text
  check_interval 0
  sentry_dsn false
  listen_address 127.0.0.1
  session_server{
    "listen_address" => "0.0.0.0:8093",
    "advertise_address" => "runner-host-name.tld:8093",
    "session_timeout"=> 1800
  }
end
```

The full resource options list is available in
[resources/global_config.rb](resources/global_config.rb).

## Changelog

Available in [CHANGELOG.md](CHANGELOG.md).

## Contributing

Please read carefully [CONTRIBUTING.md](CONTRIBUTING.md) before making a merge
request.

## License and Author

- Author:: Samuel Bernard (<samuel.bernard@gmail.com>)

```text
Copyright (c) 2015-2017 Sam4Mobile, 2017-2018 Make.org, 2019 Samuel Bernard

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
```
