# frozen_string_literal: true

# Cookbook:: gitlab-ci-runner
# Resource:: package
#
# Copyright:: 2020, Ben Hughes <bmhughes@bmhughes.co.uk>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

property :repository_url, String
property :gpgkey, String
property :gpgcheck, [true, false], default: false
property :version, String
property :package_options, Hash, default: {}
property :repository_options, Hash, default: {}

action_class do
  include GitlabCiRunner::Cookbook::Helpers
  def repo_setup(platform_family)
    url = if property_is_set?(:repository_url)
            new_resource.repo_url
          else
            repo_url
          end

    gpgkey = if property_is_set?(:gpgkey)
               new_resource.url
             else
               repo_gpg_url
             end

    case platform_family
    when 'rhel', 'amazon', 'fedora'
      r = yum_repository 'gitlab-ci-runner' do
        description 'Gitlab CI Runner'
        baseurl url
        gpgkey gpgkey
        gpgcheck new_resource.gpgcheck
      end

      new_resource.repository_options.each do |option, value|
        r.send(option, value)
      end
    when 'debian'
      %w(apt-transport-https libcurl3-gnutls lsb-release).each do |pkg|
        package "#{cookbook_name}: install #{pkg}" do
          package_name pkg
        end
      end

      r = apt_repository 'gitlab-ci-runner' do
        uri url
        distribution lazy { debian_codename }
        components ['main']
        key gpgkey
      end

      new_resource.repository_options.each do |option, value|
        r.send(option, value)
      end
    else
      raise "#{platform_family} is a unsupported platform family."
    end
  end

  def debian_codename
    Mixlib::ShellOut.new('lsb_release -c -s').run_command.stdout.chomp
  end
end

action :install do
  repo_setup(node['platform_family'])

  p = package 'gitlab-runner' do
    version new_resource.version if new_resource.version
    action :install
  end

  new_resource.package_options.each do |option, value|
    p.send(option, value)
  end
end

action :upgrade do
  repo_setup(node['platform_family'])

  p = package 'gitlab-runner' do
    action :upgrade
  end

  new_resource.package_options.each do |option, value|
    p.send(option, value)
  end
end

action :uninstall do
  package 'gitlab-runner' do
    action :remove
  end
end
