# frozen_string_literal: true

# Cookbook:: gitlab-ci-runner
# Resource:: runner
#
# Copyright:: 2020, Ben Hughes <bmhughes@bmhughes.co.uk>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

property :url, String
property :registration_token, String
property :administration_token, String
property :config, String, default: '/etc/gitlab-runner/config.toml'

# Server options
property :description, String
property :active, [true, false]
property :tag_list, Array, default: []
property :run_untagged, [true, false]
property :locked, [true, false]
property :access_level, String, equal_to: %w(not_protected ref_protected)
property :maximum_timeout, Integer

# Local options
property :options, Hash, default: {}

action_class do
  include GitlabCiRunner::Cookbook::ResourceHelpers
  include GitlabCiRunner::RunnerHelpers

  def server_setting(option)
    return new_resource.send(option.to_sym).dup if property_is_set?(option)

    return new_resource.options[option] if new_resource.options.key?(option)

    nil
  end

  def server_options
    server = {}

    GitlabCiRunner::RunnerHelpers::RUNNER_SERVER_OPTIONS.each do |option|
      setting = server_setting(option)
      next if setting.nil?

      server[option] = setting
    end

    server['description'] = new_resource.name

    server
  end

  def local_options
    local = {}

    GitlabCiRunner::RunnerHelpers::RUNNER_LOCAL_OPTIONS.each do |option|
      local[option] = new_resource.send(option.to_sym).dup
    end

    local.merge!(runner_options_to_h(new_resource.options.dup))
    local.delete_if do |option, _|
      GitlabCiRunner::RunnerHelpers::RUNNER_SERVER_OPTIONS.include?(option)
    end
  end
end

action :register do
  config_resource_init
  runners = find_resource!(:template, new_resource.config).variables['runners']

  if runner_registered?(new_resource.name,
                        local_options['url'],
                        new_resource.config)

    token, url = runner_get_token(new_resource.name, new_resource.config)
    runner_index = runner_get_index(runners, url, token)
    runner = runner_update_local(
      runners[runner_index],
      local_options.delete_if { |key, _| key.eql?('name') }
    )

    # Local configuration
    unless runner.eql?(runners[runner_index])
      converge_by("Update runner #{new_resource.name} configuration") do
        runners[runner_index] = compact_hash(runner)
      end
    end

    # Server runner
    id = runner_get_id(new_resource.name, new_resource.config)

    if property_is_set?(:administration_token)
      if runner_update_needed?(url,
                               new_resource.administration_token,
                               id,
                               server_options)
        converge_by("Updating runner #{id} on server #{url}.") do
          runner_update_server(
            url,
            new_resource.administration_token,
            id,
            server_options
          )
        end
      end
    else
      log "Unable to check/update server options for runner ID #{id} "\
          'as an administration token has not been specified.' do
        level :warn
      end
    end
  else
    converge_by("Register gitlab-runner #{new_resource.name}.") do
      # Register with Gitlab server
      id, token = runner_register(new_resource.url,
                                  new_resource.registration_token,
                                  server_options)

      # Add runner to local configuration
      new_runner = local_options
      new_runner['name'].prepend("#{id}: ")
      new_runner['token'] = token
      runners.push(compact_hash(new_runner))
    end
  end
end

action :update do
  config_resource_init

  runners = find_resource!(:template, new_resource.config).variables['runners']
  token, url = runner_get_token(new_resource.name, new_resource.config)

  return if token.nil? || url.nil?

  if runner_registered?(new_resource.name,
                        url,
                        new_resource.config)

    runner_index = runner_get_index(runners, url, token)
    runner = runner_update_local(
      runners[runner_index],
      local_options.delete_if { |key, _| key.eql?('name') }
    )

    # Local configuration
    unless runner.eql?(runners[runner_index])
      converge_by("Update runner #{new_resource.name} configuration") do
        runners[runner_index] = compact_hash(runner)
      end
    end

    # Server runner
    id = runner_get_id(new_resource.name, new_resource.config)

    if property_is_set?(:administration_token)
      if runner_update_needed?(url,
                               new_resource.administration_token,
                               id,
                               server_options)
        converge_by("Updating runner #{id} on server #{url}.") do
          runner_update_server(
            url,
            new_resource.administration_token,
            id,
            server_options
          )
        end
      end
    else
      log "Unable to check/update server options for runner ID #{id} "\
          'as an administration token has not been specified.' do
        level :warn
      end
    end
  else
    raise "Can't update unregistered runner #{new_resource.name}."
  end
end

action :unregister do
  config_resource_init
  runners = find_resource!(:template, new_resource.config).variables['runners']
  token, url = runner_get_token(new_resource.name, new_resource.config)

  unless url.nil? || token.nil?
    converge_by("Unregister gitlab-runner #{new_resource.name}") do
      if runner_registered?(
        new_resource.name,
        new_resource.url,
        new_resource.config
      )
        runner_unregister!(url, token)
      end

      index = runner_get_index(runners, url, token)
      runners.delete_at(index)
    end
  end
end
